import React from 'react';
import {observer} from "mobx-react-lite";
import {makeStyles} from "@material-ui/core/styles";
import {green, red} from "@material-ui/core/colors";
import {t} from "@lingui/macro";
import {RxModuleComponentModel} from "../../../stores/RxModuleComponentStore";
import {useSnackbar} from 'notistack';
import {CurrentUserStore} from "@rx-suite/stores/build";
import {CurrentUserModel} from "@rx-suite/stores";
import {ButtonField} from "@rx-suite/ui";
import {CloudDownload} from "@material-ui/icons";
import {LicenceInfoStore} from "../../../stores/LicenceInfoStore";

const useStyles = makeStyles(() => ({
    enabledIcon: {
        color: green[500]
    },
    warningIcon: {
        color: red[500]
    },
    errorLabel: {
        whiteSpace: 'pre-line',
        wordWrap: 'break-word'
    }
}))

type Props = {
    model: RxModuleComponentModel,
    allowReinstall?: boolean,

}

export const UpdateCell: React.FunctionComponent<Props> = observer(({model, allowReinstall = false}) => {
    const classes = useStyles();
    const {enqueueSnackbar} = useSnackbar();
    const licence = LicenceInfoStore.selectedModel;

    const onClick = React.useCallback(async e => {
        e.stopPropagation();
        if (window.confirm(`${t`Perform update now?  The module will be unavailable until the update is complete.  This may take several minutes.`} 
        
        ${t`Module:`} ${model.module_name} 
        ${t`Current Version:`} ${model.version}
        ${t`New Version:`} ${model.highestAvailableUpdate}`)) {
            enqueueSnackbar(`${model.module_name}: ${t`Update downloading & transferring`}`, {variant: "info"})
            model.update().then(resp => {
                if (resp.error) {
                    enqueueSnackbar(`${model.module_name}: ${t`Update failed`} (${resp.error_details || ''}) `, {variant: "error"})
                } else {
                    enqueueSnackbar(`${model.module_name}: ${t`Update applying - the module may take several minutes to restart`}    `, {variant: "success"})
                }
            });
        }
    }, [enqueueSnackbar, model]);

    const user = CurrentUserStore.selectedModel as unknown as CurrentUserModel;
    if (!user || !user.canModule('update', 'apply', model.module_id)) {
        return <div>{model.highestAvailableUpdate}</div>;
    }

    let text;
    if (model.reachable) {
        if(model.updateAvailable) {
            text = `${t`Update to`} ${model.highestAvailableUpdate}`;
        } else if(allowReinstall) {
            text = `${t`Reinstall`} ${model.version}`;
        } else {
            return <div>{t`Up To Date`}</div>;
        }

    } else {
        if(model.updateAvailable) {
            text = `${t`[Offline]`} ${model.highestAvailableUpdate}`;
        } else {
            return <div>{t`Up To Date`}</div>;
        }
    }

    let tooltip: string | undefined;
    if (!licence?.can_download_updates) {
        tooltip = t`Your license does not allow download updates. Please contact support.`
    }

    return (
        <ButtonField
            onClick={onClick}
            startIcon={<CloudDownload className={model.reachable ? classes.enabledIcon : classes.warningIcon}/>}
            text={text}
            variant={"outlined"}
            loading={model.is_updating}
            disabled={!model.reachable || !licence?.can_download_updates}
            tooltip={tooltip}
        />
    )
});